unit FmxUtils;

interface

uses SysUtils, WinTypes, WinProcs, Classes, Consts, LZexpand, Dialogs;

type
  EInvalidDest = class(EStreamError);
  EFCantMove = class(EStreamError);

function CopyFile(const FileName, DestName: TFileName): boolean;
function MoveFile(const FileName, DestName: TFileName): boolean;
function GetFileSize(const FileName: string): LongInt;
function FileDateTime(const FileName: string): TDateTime;
function HasAttr(const FileName: string; Attr: Word): Boolean;

implementation

const
  SInvalidDest = 'Destination %s does not exist';
  SFCantMove = 'Cannot move file %s';

function CopyFile(const FileName, DestName: TFileName): boolean;
var
  SrcHandle   : Integer;
  DestHandle  : Integer;
  S           : string;
  RetCode     : Longint;
  OpenFileBuf : TOFStruct;
  FName       : array[ 0..255 ] of Char;
begin
  StrPCopy(FName, FileName);
  SrcHandle := LZOpenFile(FName, OpenFileBuf, of_Read);
  StrPCopy(FName, DestName);
  DestHandle := LZOpenFile(FName, OpenFileBuf, of_Create);
  RetCode := LZCopy(SrcHandle, DestHandle);
  if RetCode >= 0 then
    begin
      LZClose(SrcHandle);
      LZClose(DestHandle);
      CopyFile:= true;
    end
  else
    begin
      Str(RetCode, S);
      MessageDlg('Could not copy ' + FileName + ' to ' +
        DestName + #13 + 'Error Code = ' + S,
        mtError, [mbOk], 0);
      CopyFile:= false;
    end;
end;


{ MoveFile procedure }
{
  Moves the file passed in FileName to the directory specified in DestDir.
  Tries to just rename the file.  If that fails, try to copy the file and
  delete the original.

  Raises an exception if the source file is read-only, and therefore cannot
  be deleted/moved.
}

function MoveFile(const FileName, DestName: TFileName): boolean;
begin
  MoveFile:= true;
  if not RenameFile(FileName, DestName) then { try just renaming }
  begin
    MoveFile:= false;
    if HasAttr(FileName, faReadOnly) then  { if it's read-only... }
      raise EFCantMove.Create(Format(SFCantMove, [FileName])); { we wouldn't be able to delete it }
    if CopyFile(FileName, DestName) = false then exit; { copy it over to destination...}
    if DeleteFile(FileName) = false then exit; { ...and delete the original }
    MoveFile:= true;
  end;
end;

{ GetFileSize function }
{
  Returns the size of the named file without opening the file.  If the file
  doesn't exist, returns -1.
}

function GetFileSize(const FileName: string): LongInt;
var
  SearchRec: TSearchRec;
begin
  if FindFirst(ExpandFileName(FileName), faAnyFile, SearchRec) = 0 then
    Result := SearchRec.Size
  else Result := -1;
end;

function FileDateTime(const FileName: string): System.TDateTime;
begin
  Result := FileDateToDateTime(FileAge(FileName));
end;

function HasAttr(const FileName: string; Attr: Word): Boolean;
begin
  Result := (FileGetAttr(FileName) and Attr) = Attr;
end;

end.
