unit Main;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DirSpec, StdCtrls, Gauges, ExtCtrls, Menus,
  FmxUtils, About, Buttons, Grids, Printers;

const
  FieldSize = 44;

type
  TMainForm = class(TForm)
    SpeedPanel: TPanel;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    CopySelected: TMenuItem;
    MoveSelected: TMenuItem;
    DeleteSelected: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    Contents1: TMenuItem;
    N2: TMenuItem;
    About1: TMenuItem;
    SpeedDeleteBtn: TSpeedButton;
    SpeedMoveBtn: TSpeedButton;
    SpeedCopyBtn: TSpeedButton;
    SpeedSwapBtn: TSpeedButton;
    Directory1: TMenuItem;
    FilesPanel: TPanel;
    AllFiles: TRadioButton;
    DifferentFiles: TRadioButton;
    AlikeFiles: TRadioButton;
    TargetDirectory1: TMenuItem;
    Panel1: TPanel;
    SourcePanel: TPanel;
    SpeedSourceBtn: TSpeedButton;
    TargetPanel: TPanel;
    SpeedTargetBtn: TSpeedButton;
    StringGrid1: TStringGrid;
    PrintDialog1: TPrintDialog;
    PrinterSetupDialog1: TPrinterSetupDialog;
    PrintListing1: TMenuItem;
    PrinterSetup1: TMenuItem;
    N3: TMenuItem;
    SaveDialog1: TSaveDialog;
    Swap: TMenuItem;
    Panel4: TPanel;
    Panel2: TPanel;
    SourceFiles: TPanel;
    TargetFiles: TPanel;
    Gauge1: TGauge;
    SpeedMarkBtn: TSpeedButton;
    SpeedUnMarkBtn: TSpeedButton;
    MarkAll1: TMenuItem;
    UnMarkAll1: TMenuItem;
    procedure LoadDir(FileName: string);
    procedure MatchAllFiles;
    procedure MatchDifferentFiles;
    procedure MatchAlikeFiles;
    procedure Match;
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SourceDirectoryBtnClick(Sender: TObject);
    procedure TargetDirectoryBtnClick(Sender: TObject);
    procedure SwapBtnClick(Sender: TObject);
    procedure AllFilesClick(Sender: TObject);
    procedure DifferentFilesClick(Sender: TObject);
    procedure AlikeFilesClick(Sender: TObject);
    procedure CopyMove(CopyMoveCaption: string);
    procedure CopySelectedClick(Sender: TObject);
    procedure MoveSelectedClick(Sender: TObject);
    procedure DeleteSelectedClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Contents1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure StringGrid1DrawCell(Sender: TObject; Col, Row: Longint;
      Rect: TRect; State: TGridDrawState);
    procedure StringGrid1DblClick(Sender: TObject);
    procedure StringGrid1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PrintListing1Click(Sender: TObject);
    procedure PrinterSetup1Click(Sender: TObject);
    procedure SpeedMarkBtnClick(Sender: TObject);
    procedure SpeedUnMarkBtnClick(Sender: TObject);
  private
    { Private declarations }
    SourceList : TStringList;
    TargetList : TStringList;
    FileList   : TStringList;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

type
  TTimeField = LongInt;

procedure TMainForm.LoadDir(FileName: string);
var
  SearchRec : TSearchRec;
  ReturnValue : integer;

  procedure MakeName;
  var
    FileName : string;
    FileNamePtr : pointer;
    FileSize : longint;
    FileTime : longint;
  begin
    with SearchRec do
    begin
    if Attr and (faHidden or faSysFile
      or faDirectory or faVolumeId) <> 0 then exit;
    FileName:= ' ' + LowerCase(Name);
    FillChar(FileName[length(FileName) + 1],
      FieldSize - length(FileName), ' ');
    FileName[0]:= char(FieldSize);
    FileNamePtr:= @FileName;
    FileSize:= Size;
    FileTime:= Time;
    asm
      les   di,FileNamePtr
      add   di,16
      push  di
      add   di,8
      mov   dx,word ptr FileSize[0]
      mov   ax,word ptr FileSize[2]

      std
      mov   bx,10
    @next_size:
      mov   cx,dx
      xor   dx,dx
      div   bx
      xchg  ax,cx
      div   bx
      xchg  ax,dx
      add   al,'0'
      stosb
      mov   ax,cx
      or    cx,dx
      jnz   @next_size

      cld
      pop   di
      add   di,11

    @store_date:
      mov   dx,word ptr FileTime[2]
      mov   ax,dx
      mov   cl,5
      shr   ax,cl
      and   ax,1111b
      div   bl
      add   ax,'00'
      cmp   al,'0'
      jnz   @store_it1
      mov   al,' '
    @store_it1:
      stosw
      mov   al,'/'
      stosb

      mov   ax,dx
      and   ax,11111b
      div   bl
      add   ax,'00'
      stosw
      mov   al,'/'
      stosb

      mov   ax,dx
      mov   cl,9
      shr   ax,cl
      add   ax,80
      cmp   ax,100
      jb    @display_date
      sub   ax,100
    @display_date:
      mov   ch,' '
      div   bl
      add   ax,'00'
      stosw
      mov   al,' '
      stosb

    @time:
      inc   di
      mov   dx,word ptr FileTime[0]
      mov   ax,dx
      mov   cl,11
      shr   ax,cl
      push  ax
      cmp   ax,12
      jbe   @meridian
      sub   ax,12
    @meridian:
      or    ax,ax
      jnz   @not_midnight
      mov   ax,12
    @not_midnight:
      div   bl
      add   ax,'00'
      cmp   al,'0'
      jnz   @store_it2
      mov   al,' '
    @store_it2:
      stosw
      mov   al,':'
      stosb

      mov   ax,dx
      mov   cl,5
      shr   ax,cl
      and   ax,111111b
      div   bl
      add   ax,'00'
      stosw

      mov   al,'p'
      pop   dx
      cmp   dx,12
      jae   @pm
      mov   al,'a'
    @pm:
      mov   ah,'m'
      stosw
    end;
      FileList.AddObject(FileName, Pointer(FileTime));
    end;
  end;

begin
  if FindFirst(FileName, faAnyFile, SearchRec) = 0 then MakeName;
    repeat
      ReturnValue:= FindNext(SearchRec);
      if ReturnValue = 0 then MakeName;
    until ReturnValue <> 0;
  Match;
end;

procedure TMainForm.MatchAllFiles;
var
  i, j : integer;
  TempName1, TempName2 : string;
begin
  i:= 0;
  j:= 0;
  with StringGrid1 do
  begin
    while (i < SourceList.Count) and (j < TargetList.Count) do
    begin
      RowCount:= RowCount + 1;
      TempName1:= Copy(SourceList[i], 2, 12);
      TempName2:= Copy(TargetList[j], 2, 12);
      case AnsiCompareStr(TempName1, TempName2) of
      -32768..-1 :  begin
                      Cols[1].Add(SourceList[i]);
                      Cols[3].Add(' ');
                      inc(i);
                    end;
      0          :  begin
                      Cols[1].AddObject(SourceList[i],
                        SourceList.Objects[i]);
                      inc(i);
                      Cols[3].AddObject(TargetList[j],
                        TargetList.Objects[j]);
                      inc(j);
                    end;
      1..32767   :  begin
                      Cols[3].Add(TargetList[j]);
                      Cols[1].Add(' ');
                      inc(j);
                    end;
      end;
    end;

    while (i < SourceList.Count) do
    begin
      RowCount:= RowCount + 1;
      Cols[1].Add(SourceList[i]);
      Cols[3].Add(' ');
      inc(i);
    end;

    while (j < TargetList.Count) do
    begin
      RowCount:= RowCount + 1;
      Cols[3].Add(TargetList[j]);
      Cols[1].Add(' ');
      inc(j);
    end;

    RowCount:= RowCount - 1;
  end;
end;

procedure TMainForm.MatchDifferentFiles;
var
  i, j : integer;
  TempName1, TempName2 : string;
  AObj, BObj : TTimeField;
begin
  i:= 0;
  j:= 0;
  with StringGrid1 do
  begin
    while (i < SourceList.Count) and (j < TargetList.Count) do
    begin
      TempName1:= Copy(SourceList[i], 2, 12);
      TempName2:= Copy(TargetList[j], 2, 12);
      case AnsiCompareStr(TempName1, TempName2) of
      -32768..-1 :  begin
                      RowCount:= RowCount + 1;
                      Cols[1].Add(SourceList[i]);
                      Cols[3].Add(' ');
                      inc(i);
                    end;
      0          :  begin
                      AObj:= TTimeField(SourceList.Objects[i]);
                      BObj:= TTimeField(TargetList.Objects[j]);
                      if AObj = BObj then
                        begin
                          inc(i);
                          inc(j);
                        end
                      else
                        begin
                          RowCount:= RowCount + 1;
                          Cols[1].AddObject(SourceList[i],
                            SourceList.Objects[i]);
                          inc(i);
                          Cols[3].AddObject(TargetList[j],
                            TargetList.Objects[j]);
                          inc(j);
                        end;
                    end;
      1..32767   :  begin
                      RowCount:= RowCount + 1;
                      Cols[3].Add(TargetList[j]);
                      Cols[1].Add(' ');
                      inc(j);
                    end;
      end;
    end;

    while (i < SourceList.Count) do
    begin
      RowCount:= RowCount + 1;
      Cols[1].Add(SourceList[i]);
      Cols[3].Add(' ');
      inc(i);
    end;

    while (j < TargetList.Count) do
    begin
      RowCount:= RowCount + 1;
      Cols[3].Add(TargetList[j]);
      Cols[1].Add(' ');
      inc(j);
    end;

    RowCount:= RowCount - 1;
  end;
end;

procedure TMainForm.MatchAlikeFiles;
var
  i, j : integer;
  TempName1, TempName2 : string;
  AObj, BObj : TTimeField;
begin
  i:= 0;
  j:= 0;
  with StringGrid1 do
  begin
    while (i < SourceList.Count) and (j < TargetList.Count) do
    begin
      TempName1:= Copy(SourceList[i], 2, 12);
      TempName2:= Copy(TargetList[j], 2, 12);
      case AnsiCompareStr(TempName1, TempName2) of
      -32768..-1 :  begin
                      inc(i);
                    end;
      0          :  begin
                      RowCount:= RowCount + 1;
                      Cols[1].AddObject(SourceList[i],
                        SourceList.Objects[i]);
                      inc(i);
                      Cols[3].AddObject(TargetList[j],
                        TargetList.Objects[j]);
                      inc(j);
                    end;
      1..32767   :  begin
                      inc(j);
                    end;
      end;
    end;

    RowCount:= RowCount - 1;
  end;
end;

procedure TMainForm.Match;
var
  i, j: integer;
begin
  with StringGrid1 do
  begin
    AllFiles.Cursor:= crHourGlass;
    DifferentFiles.Cursor:= crHourGlass;
    AlikeFiles.Cursor:= crHourGlass;
    Cursor:= crHourGlass;
    for i:= 0 to 3 do Cols[i].Clear;
    RowCount:= 0;
    if AllFiles.Checked = true then MatchAllFiles;
    if DifferentFiles.Checked = true then MatchDifferentFiles;
    if AlikeFiles.Checked = true then MatchAlikeFiles;
    SetFocus;
    Row:= 0;
    Col:= 0;

    j:= 0;
    for i:= 0 to RowCount - 1 do
      if (Cells[1, i] <> ' ') and (Cells[1, i] <> '') then inc(j);
    SourceFiles.Caption:= IntToStr(j) + ' File(s)';

    j:= 0;
    for i:= 0 to RowCount - 1 do
      if (Cells[3, i] <> ' ') and (Cells[1, i] <> '') then inc(j);
    TargetFiles.Caption:= IntToStr(j) + ' File(s)';

    FormResize(Self);

    AllFiles.Cursor:= crDefault;
    DifferentFiles.Cursor:= crDefault;
    AlikeFiles.Cursor:= crDefault;
    Cursor:= crDefault;
  end;
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  with StringGrid1 do
  begin
    ColWidths[1]:= (ClientWidth - DefaultColWidth - 3) Div 2;
    if ScrollBars = ssBoth then
      ColWidths[1]:= ColWidths[1] - 10;
    ColWidths[3]:= ColWidths[1];

    SourcePanel.Width:= ColWidths[1] + DefaultColWidth + 2;
    TargetPanel.Left:= SourcePanel.Width;
    TargetPanel.Width:= ClientWidth - SourcePanel.Width;

    SourceFiles.Width:= ColWidths[1] + DefaultColWidth + 2;
    TargetFiles.Left:= SourcePanel.Width;
    TargetFiles.Width:= ClientWidth - SourcePanel.Width;
  end;
end;

procedure TMainForm.SourceDirectoryBtnClick(Sender: TObject);
begin
  with SourcePanel do
  begin
    if Caption <> '' then
    begin
      Dir.DirectoryListBox1.Directory:= ExtractFilePath(Caption);
      Dir.FileSpecMask:= ExtractFileName(Caption);
    end;
    if Dir.ShowModal <> mrOK then exit;
    Caption:= Dir.DirectoryListBox1.Directory;
    if Caption[length(Caption)] <> '\' then
      Caption:= Caption + '\';
    Caption:= Caption + Dir.FileSpecMask;
    SourceList.Clear;
    FileList:= SourceList;
    LoadDir(Caption);
    FormResize(Sender);
  end;
end;

procedure TMainForm.TargetDirectoryBtnClick(Sender: TObject);
begin
  with TargetPanel do
  begin
    if Caption <> '' then
    begin
      Dir.DirectoryListBox1.Directory:= ExtractFilePath(Caption);
      Dir.FileSpecMask:= ExtractFileName(Caption);
    end;
    if Dir.ShowModal <> mrOK then exit;
    Caption:= Dir.DirectoryListBox1.Directory;
    if Caption[length(Caption)] <> '\' then
      Caption:= Caption + '\';
    Caption:= Caption + Dir.FileSpecMask;
    TargetList.Clear;
    FileList:= TargetList;
    LoadDir(Caption);
    FormResize(Sender);
  end;
end;

procedure TMainForm.SwapBtnClick(Sender: TObject);
var
  i : integer;
  TempList, TempList2 : TStringList;
  TempName : string;
  TempCols, TempCols2 : TStringList;
begin
  with StringGrid1 do
  begin
    TempList:= TStringList.Create;
    try
      TempList2:= TStringList.Create;
      try
        TempCols:= TStringList.Create;
        try
          TempCols2:= TStringList.Create;
          try
            for i:= 0 to SourceList.Count - 1 do
              TempList.AddObject(SourceList[i],
                SourceList.Objects[i]);
            TempName:= SourcePanel.Caption;
            SourceList.Assign(TargetList);
            SourcePanel.Caption:= TargetPanel.Caption;
            TargetList.Assign(TempList);
            TargetPanel.Caption:= TempName;
            TempList.Clear;

            for i:= 0 to RowCount - 1 do
            begin
              TempCols.Add(Cells[0, i]);
              TempCols2.Add(Cells[2, i]);
              TempList.AddObject(Cells[1, i], Objects[1, i]);
              TempList2.AddObject(Cells[3, i], Objects[3, i]);
            end;

            for i:= 0 to 3 do Cols[i].Clear;

            Cols[0].Assign(TempCols2);
            Cols[2].Assign(TempCols);
            Cols[1].Assign(TempList2);
            Cols[3].Assign(TempList);
          finally
            TempCols2.Free;
          end;
        finally
          TempCols.Free;
        end;
      finally
        TempList2.Free;
      end;
    finally
      TempList.Free;
    end;
  end;
end;

procedure TMainForm.AllFilesClick(Sender: TObject);
begin
  Match;
end;

procedure TMainForm.DifferentFilesClick(Sender: TObject);
begin
  Match;
end;

procedure TMainForm.AlikeFilesClick(Sender: TObject);
begin
  Match;
end;

procedure TMainForm.CopyMove(CopyMoveCaption: string);
var
  i, k: integer;
  Source, Target, TempName: string;
begin
  Source:= SourcePanel.Caption;
  Target:= TargetPanel.Caption;

  if (Source = '') or (Target = '') then
  begin
    MessageDlg('Two directories not open.',
      mtInformation, [mbOk], 0);
    exit;
  end;

  while (Length(Source) <> 0) and
    (Source[Length(Source)] <> '\') do dec(Source[0]);
  while (Length(Target) <> 0) and
    (Target[Length(Target)] <> '\') do dec(Target[0]);

  if Source = Target then
  begin
    MessageDlg('Can''t copy or move to same directory',
      mtInformation, [mbOk], 0);
    exit;
  end;

  with StringGrid1 do
  begin
    i:= 0;
    for k:= 0 to RowCount - 1 do
      if Cells[0, k] = #252 then inc(i);
    if i = 0 then
    begin
      MessageDlg('Nothing check marked', mtInformation, [mbok], 0);
      exit;
    end;

    if MessageDlg(CopyMoveCaption + ' ' + IntToStr(i) +
      ' check marked file(s) from' + #13 +
      Source + #13 + ' to' + #13 + Target,
        mtConfirmation, mbOkCancel, 0) <> mrok then exit;

    Gauge1.MaxValue:=i;
    Gauge1.Visible:= true;

    for k:= 0 to RowCount - 1 do
    begin
      if Cells[0, k] = #252 then
      begin
        TempName:= Cells[1, k];
        Delete(TempName, 1, 1);
        TempName[0]:= Char(Pos(' ', TempName) - 1);

        if (FileGetAttr(Target + TempName) and faReadOnly = faReadOnly) or
          ((CopyMoveCaption = 'Move') and
            (FileGetAttr(Source + TempName) and faReadOnly = faReadOnly)) then
              MessageDlg('Can not Copy or Move ReadOnly file ' + Source + TempName +
                ' to ' + Target + TempName,
                  mtInformation, [mbok], 0)
        else
        begin
          if CopyMoveCaption = 'Copy' then
            begin
              if CopyFile(Source + TempName,
                Target + TempName) = false then break;
            end
          else
            begin
              if MoveFile(Source + TempName,
                Target + TempName) = false then break;
            end;
        end;

        Gauge1.Progress:= Gauge1.Progress + 1;
        dec(i);
        if i = 0 then break;
      end;
    end;
  end;

  Gauge1.Progress:=0;
  Gauge1.Visible:= false;

  SourceList.Clear;
  FileList:= SourceList;
  LoadDir(SourcePanel.Caption);

  TargetList.Clear;
  FileList:= TargetList;
  LoadDir(TargetPanel.Caption);
end;

procedure TMainForm.CopySelectedClick(Sender: TObject);
begin
  CopyMove('Copy');
end;

procedure TMainForm.MoveSelectedClick(Sender: TObject);
begin
  CopyMove('Move');
end;

procedure TMainForm.DeleteSelectedClick(Sender: TObject);
var
  i, k: integer;
  Source, TempName: string;
begin
  Source:= SourcePanel.Caption;

  while (Length(Source) <> 0) and
    (Source[Length(Source)] <> '\') do dec(Source[0]);

  with StringGrid1 do
  begin
    i:= 0;
    for k:= 0 to RowCount - 1 do
      if Cells[0, k] = #252 then inc(i);
    if i = 0 then
    begin
      MessageDlg('Nothing check marked', mtInformation, [mbok], 0);
      exit;
    end;

    if MessageDlg('Delete ' + IntToStr(i) +
      ' check marked file(s) from' + #13 + Source,
        mtConfirmation, mbOkCancel, 0) <> mrok then exit;

    Gauge1.MaxValue:=i;
    Gauge1.Visible:= true;

    for k:= 0 to RowCount - 1 do
    begin
      if Cells[0, k] = #252 then
      begin
        TempName:= Cells[1, k];
        Delete(TempName, 1, 1);
        TempName[0]:= Char(Pos(' ', TempName) - 1);

        if (FileGetAttr(Source + TempName) and faReadOnly = faReadOnly) or
          (DeleteFile(Source + TempName) = false) then
            MessageDlg('Could not delete ' + Source
              + TempName, mtInformation, [mbok], 0);

        Gauge1.Progress:= Gauge1.Progress + 1;
        dec(i);
        if i = 0 then break;
      end;
    end;
  end;

  Gauge1.Progress:=0;
  Gauge1.Visible:= false;

  SourceList.Clear;
  FileList:= SourceList;
  LoadDir(SourcePanel.Caption);

  TargetList.Clear;
  FileList:= TargetList;
  LoadDir(TargetPanel.Caption);
end;

procedure TMainForm.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.Contents1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_CONTENTS,0);
end;

procedure TMainForm.About1Click(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  StringGrid1.ColWidths[2]:= 0;
  FormResize(Sender);
  Dir.FileSpecMask:= '*.*';
  SourceList:= TStringList.Create;
  TargetList:= TSTringList.Create;
  SourceList.Sorted:= true;
  TargetList.Sorted:= true;
  StringGrid1.SetFocus;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  SourceList.Free;
  TargetList.Free;
end;

procedure TMainForm.StringGrid1DrawCell(Sender: TObject; Col, Row: Longint;
  Rect: TRect; State: TGridDrawState);
var
  vCol, vRow : longint;
  AObj, BObj : TTimeField;
begin
  vCol:= Col;
  vRow:= Row;
  with Sender as TStringGrid, Canvas do
  begin
    if (odd(vCol)) then
      begin
        if (length(StringGrid1.Cells[1, vRow]) < 2) or
           (length(StringGrid1.Cells[3, vRow]) < 2) then exit;

        AObj:= TTimeField(StringGrid1.Objects[vCol, vRow]);
        BObj:= TTimeField(StringGrid1.Objects[(VCol + 2) Mod 4, vRow]);
        if AObj <= BObj then exit;

        FillRect(Rect);
        Font.Style:= [fsBold, fsItalic];
        Font.Color:= clBlue;
        TextOut(Rect.Left, Rect.Top, Cells[vCol, vRow]);
      end
    else
      begin
        Font.Name:= 'WingDings';
        Font.Pitch:= fpDefault;
        Font.Height:= -15;
        TextOut(Rect.Left + 1, Rect.Top + 2, Cells[vCol, vRow]);
      end;
  end;
end;

procedure TMainForm.StringGrid1DblClick(Sender: TObject);
begin
  with Sender as TStringGrid do
  begin
    if length(Cells[1, Row]) < 2 then exit;
    if Cells[0, Row] = #252 then
      Cells[0, Row]:= ' '
    else
      Cells[0, Row]:= #252;
  end;
end;

procedure TMainForm.StringGrid1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  with StringGrid1 do
  case Key of
  VK_SPACE: begin
              StringGrid1DblClick(Sender);
              if Shift = [ssShift] then
              begin
                if Row > 0 then Row:= Row - 1
              end
              else
                if Row < Rowcount - 1 then Row:= Row + 1;
            end;

  VK_LEFT:  if Col = 3 then Col:= 2;

  VK_RIGHT: if Col = 1 then Col:= 2;
  end;
end;

procedure TMainForm.PrintListing1Click(Sender: TObject);
var
  F : TextFile;
  i : integer;
  AObj, BObj : TTimeField;
  Line : integer;
  PrintText : System.Text;
const
  Blank : string[FieldSize] = '                                            ';
begin
  if PrintDialog1.Execute then
  begin
    with StringGrid1 do
    begin
      if PrintDialog1.PrintToFile = true then
      begin
        with SaveDialog1 do
        begin
          if Execute then
          begin
            if FileExists(FileName) then
              if MessageDlg('Overwrite ' + FileName + '?',
                mtConfirmation, [mbOk, mbCancel], 0) = mrCancel then exit;
            AssignFile(F, FileName);
            ReWrite(F);
            Write(F, SourcePanel.Caption);
            for i:= 0 to length(Blank) - length(SourcePanel.Caption) do
              Write(F, ' ');
            Writeln(F, #9 + TargetPanel.Caption);
            Writeln(F, '');

            for Line:= 0 to RowCount - 1 do
            begin
              if length(Cells[1, Line]) < 2 then
                Write(F, Blank)
              else
                Write(F, Cells[1, Line]);
              if length(Cells[3, Line]) < 2 then
                Writeln(F, #9 + Blank)
              else
              Writeln(F, #9 + Cells[3, Line]);
            end;
            CloseFile(F);
          end;
          exit;
        end;
      end;

      AssignPrn(PrintText);
      Rewrite(PrintText);
      Printer.Canvas.Font:= Font;
      Printer.Canvas.Font.Size:= 9;

      Write(PrintText, #9 + SourcePanel.Caption);
      for i:= 0 to length(Blank) - length(SourcePanel.Caption) do
        Write(PrintText, ' ');
      Writeln(PrintText, #9 + TargetPanel.Caption);

      Writeln(PrintText, '');

      for Line:= 0 to RowCount - 1 do
      begin
        if (length(Cells[1, Line]) < 2) or
           (length(Cells[3, Line]) < 2) then
          begin
            if length(Cells[1, Line]) < 2 then
              Write(PrintText, #9 + Blank)
            else
              Write(PrintText, #9 + Cells[1, Line]);
            if length(Cells[3, Line]) < 2 then
              Writeln(PrintText, #9 + Blank)
            else
              Writeln(PrintText, #9 + Cells[3, Line]);
          end
        else
          begin
            AObj:= TTimeField(Objects[1, Line]);
            BObj:= TTimeField(Objects[3, Line]);
            if AObj < BObj then
              Printer.Canvas.Font.Style:= [fsBold, fsItalic];
            Write(PrintText, #9 + Cells[1, Line]);
              Printer.Canvas.Font.Style:= [fsBold];
            if AObj > BObj then
              Printer.Canvas.Font.Style:= [fsBold, fsItalic];
            Writeln(PrintText, #9 + Cells[3, Line]);
              Printer.Canvas.Font.Style:= [fsBold];
          end;
      end;
      CloseFile(PrintText);
    end;
  end;
end;

procedure TMainForm.PrinterSetup1Click(Sender: TObject);
begin
  PrinterSetupDialog1.Execute;
end;

procedure TMainForm.SpeedMarkBtnClick(Sender: TObject);
var
  i : integer;
begin
  with StringGrid1 do
    for i:= 0 to RowCount - 1 do
      if (Cells[1, i] <> ' ') and
      (Cells[1, i] <> '') then Cells[0, i]:= #252;
end;

procedure TMainForm.SpeedUnMarkBtnClick(Sender: TObject);
var
  i : integer;
begin
  with StringGrid1 do
    for i:= 0 to RowCount - 1 do
    begin
      Cells[0, i]:= '';
      Cells[2, i]:= '';
    end;
end;

end.
